@@ -5,6 +5,7 @@ import logging
 import socket
 import time
 import ssl
+import sys
 
 from multiprocessing import Process, Manager, Queue, pool
 from threading import RLock, Thread
@@ -31,17 +32,17 @@ from .files import read_file, save_file, delete_file
 logger = logging.getLogger("acm")
 
 DEBUG = False
-VERSION = "0.1.8"
+VERSION = "0.2.0"
 
 DEFAULT_GROUP_NAME = "DEFAULT_GROUP"
-WORD_SEPARATOR = bytes([2]).decode()
-LINE_SEPARATOR = bytes([1]).decode()
+DEFAULT_NAMESPACE = ""
+
+WORD_SEPARATOR = u'\x02'
+LINE_SEPARATOR = u'\x01'
 
 DEFAULTS = {
     "APP_NAME": "ACM-SDK-Python",
     "TIMEOUT": 3,  # in seconds
-    "GROUP": "DEFAULT_GROUP",
-    "NAMESPACE": "DEFAULT_TENANT",
     "PULLING_TIMEOUT": 30,  # in seconds
     "PULLING_CONFIG_SIZE": 3000,
     "CALLBACK_THREAD_NUM": 10,
@@ -49,8 +50,9 @@ DEFAULTS = {
     "SNAPSHOT_BASE": "acm-data/snapshot"
 }
 
-OPTIONS = {"default_timeout", "tls_enabled", "auth_enabled", "cai_enabled", "pulling_timeout", "pulling_config_size",
-           "callback_thread_num", "failover_base", "snapshot_base", "app_name"}
+OPTIONS = set(
+    ["default_timeout", "tls_enabled", "auth_enabled", "cai_enabled", "pulling_timeout", "pulling_config_size",
+     "callback_thread_num", "failover_base", "snapshot_base", "app_name"])
 
 
 class ACMException(Exception):
@@ -74,7 +76,15 @@ def process_common_params(data_id, group):
 def parse_pulling_result(result):
     if not result:
         return list()
-    return [i.split(WORD_SEPARATOR) for i in unquote_plus(result.decode(), "utf8").split(LINE_SEPARATOR) if i.strip()]
+    ret = list()
+    for i in unquote_plus(result.decode()).split(LINE_SEPARATOR):
+        if not i.strip():
+            continue
+        sp = i.split(WORD_SEPARATOR)
+        if len(sp) < 3:
+            sp.append("")
+        ret.append(sp)
+    return ret
 
 
 class WatcherWrap:
@@ -89,7 +99,8 @@ class CacheData:
         self.key = key
         local_value = read_file(client.failover_base, key) or read_file(client.snapshot_base, key)
         self.content = local_value
-        self.md5 = hashlib.md5(local_value.encode()).hexdigest() if local_value else None
+        src = local_value.decode("utf8") if type(local_value) == bytes else local_value
+        self.md5 = hashlib.md5(src.encode("GBK")).hexdigest() if src else None
         self.is_init = True
         if not self.md5:
             logger.debug("[init-cache] cache for %s does not have local value" % key)
@@ -117,7 +128,7 @@ class ACMClient:
 
     def __init__(self, endpoint, namespace=None, ak=None, sk=None, ):
         self.endpoint = endpoint
-        self.namespace = namespace or DEFAULTS["NAMESPACE"]
+        self.namespace = namespace or DEFAULT_NAMESPACE or ""
         self.ak = ak
         self.sk = sk
 
@@ -154,7 +165,7 @@ class ACMClient:
                 continue
 
             logger.debug("[set_options] key:%s, value:%s" % (k, v))
-            self.__setattr__(k, v)
+            setattr(self, k, v)
 
     def _refresh_server_list(self):
         with self.server_list_lock:
@@ -198,13 +209,39 @@ class ACMClient:
                 self.server_list = server_list
                 self.current_server = self.server_list[self.server_offset]
                 logger.info("[get-server] server_num:%s server_list:%s" % (len(self.server_list), self.server_list))
-            t = Thread(target=self._refresh_server_list)
-            t.setDaemon(True)
-            t.start()
+
+            if self.cai_enabled:
+                t = Thread(target=self._refresh_server_list)
+                t.setDaemon(True)
+                t.start()
 
         logger.info("[get-server] use server:%s" % str(self.current_server))
         return self.current_server
 
+    def _publish(self, data_id, group, content, timeout=None):
+        # todo publish API
+        if content is None:
+            raise ACMException("Can not publish none, use remove instead.")
+
+        data_id, group = process_common_params(data_id, group)
+        logger.info("[publish] data_id:%s, group:%s, namespace:%s, content:%s, timeout:%s" % (
+            data_id, group, self.namespace, truncate(content), timeout))
+
+        params = {
+            "dataId": data_id,
+            "group": group,
+            "content": content,
+        }
+        if self.namespace:
+            params["tenant"] = self.namespace
+        try:
+            data = urlencode(params, encoding="GBK").encode()
+            resp = self._do_sync_req("/diamond-server/basestone.do?method=syncUpdateAll", None, None, data,
+                                     timeout or self.default_timeout)
+            d = resp.read()
+        except:
+            logger.exception("xxx")
+
     def get(self, data_id, group, timeout=None):
         """Get value of one config item.
 
@@ -230,8 +267,9 @@ class ACMClient:
         params = {
             "dataId": data_id,
             "group": group,
-            "tenant": self.namespace
         }
+        if self.namespace:
+            params["tenant"] = self.namespace
 
         cache_key = group_key(data_id, group, self.namespace)
         # get from failover
@@ -246,11 +284,6 @@ class ACMClient:
         try:
             resp = self._do_sync_req("/diamond-server/config.co", None, params, None, timeout or self.default_timeout)
             content = resp.read().decode("GBK")
-            logger.info(
-                "[get-config] content from server:%s, data_id:%s, group:%s, namespace:%s, try to save snapshot" % (
-                    truncate(content), data_id, group, self.namespace))
-            save_file(self.snapshot_base, cache_key, content)
-            return content
         except HTTPError as e:
             if e.code == HTTPStatus.NOT_FOUND:
                 logger.warning(
@@ -274,6 +307,17 @@ class ACMClient:
         except Exception as e:
             logger.exception("[get-config] exception %s occur" % str(e))
 
+        if content is not None:
+            logger.info(
+                "[get-config] content from server:%s, data_id:%s, group:%s, namespace:%s, try to save snapshot" % (
+                    truncate(content), data_id, group, self.namespace))
+            try:
+                save_file(self.snapshot_base, cache_key, content)
+            except Exception as e:
+                logger.exception("[get-config] save snapshot failed for %s, data_id:%s, group:%s, namespace:%s" % (
+                    data_id, group, self.namespace, str(e)))
+            return content
+
         logger.error("[get-config] get config from server failed, try snapshot, data_id:%s, group:%s, namespace:%s" % (
             data_id, group, self.namespace))
         content = read_file(self.snapshot_base, cache_key)
@@ -398,19 +442,25 @@ class ACMClient:
                 address, port, is_ip_address = server_info
                 server = ":".join([address, str(port)])
                 # if tls is enabled and server address is in ip, turn off verification
-                if self.tls_enabled and is_ip_address:
-                    context = ssl.SSLContext(ssl.PROTOCOL_SSLv23)
-                    context.check_hostname = False
-                else:
-                    context = None
+
                 server_url = "%s://%s" % ("https" if self.tls_enabled else "http", server)
                 req = Request(url=server_url + url, data=data, headers=all_headers)
-                resp = urlopen(req, timeout=timeout, context=context)
+
+                # for python2.6 compatibility
+                if sys.version_info[0] == 2 and sys.version_info[1] == 6:
+                    resp = urlopen(req, timeout=timeout)
+                else:
+                    if self.tls_enabled and is_ip_address:
+                        context = ssl.SSLContext(ssl.PROTOCOL_SSLv23)
+                        context.check_hostname = False
+                    else:
+                        context = None
+                    resp = urlopen(req, timeout=timeout, context=context)
                 logger.debug("[do-sync-req] info from server:%s" % server)
                 return resp
             except HTTPError as e:
-                if e.code in {HTTPStatus.INTERNAL_SERVER_ERROR, HTTPStatus.BAD_GATEWAY,
-                              HTTPStatus.SERVICE_UNAVAILABLE}:
+                if e.code in [HTTPStatus.INTERNAL_SERVER_ERROR, HTTPStatus.BAD_GATEWAY,
+                              HTTPStatus.SERVICE_UNAVAILABLE]:
                     logger.warning("[do-sync-req] server:%s is not available for reason:%s" % (server, e.msg))
                 else:
                     raise
@@ -462,7 +512,7 @@ class ACMClient:
             changed_keys = list()
             try:
                 resp = self._do_sync_req("/diamond-server/config.co", headers, None, data, self.pulling_timeout + 10)
-                changed_keys = {group_key(*i) for i in parse_pulling_result(resp.read())}
+                changed_keys = [group_key(*i) for i in parse_pulling_result(resp.read())]
                 logger.debug("[do-pulling] following keys are changed from server %s" % truncate(str(changed_keys)))
             except ACMException as e:
                 logger.error("[do-pulling] acm exception: %s" % str(e))
@@ -474,8 +524,9 @@ class ACMClient:
                 if cache_key in changed_keys:
                     data_id, group, namespace = parse_key(cache_key)
                     content = self.get(data_id, group)
-                    md5 = hashlib.md5(content.encode()).hexdigest() if content is not None else None
+                    md5 = hashlib.md5(content.encode("GBK")).hexdigest() if content is not None else None
                     cache_data.md5 = md5
+                    cache_data.content = content
                 queue.put((cache_key, cache_data.content, cache_data.md5))
 
     @synchronized_with_attr("pulling_lock")
